@@ -10,9 +10,24 @@ batch_len=${2:-5000000}
 
 qlen=`mktemp`
 output_dir=`mktemp -d`
-trap " /bin/rm -fr $output_dir $qlen " INT QUIT EXIT HUP KILL ALRM
+tmp_dir=`mktemp -d`
+single_fasta=`mktemp -t $(basename -s .sh $0)-XXXXXXX.fa`
+SCRIPT_DIR=$(cd "`dirname "$0"`"; pwd)
+trap " /bin/rm -fr $single_fasta $tmp_dir $output_dir $qlen " INT QUIT EXIT HUP KILL ALRM
 
-bin/fasta_split.py -l $batch_len -o $output_dir -c $qlen $query
+if [[ $query =~ ".query-list" ]]; then
+    echo "Processing .query-list file"
+    if [[ $query =~ "s3://" ]]; then
+        aws s3 cp --only-show-errors $query - | parallel aws s3 cp --only-show-errors {} $output_dir/
+    else
+        gsutil -qm cat $query | parallel gsutil -qm cp {} $output_dir/
+    fi
+    parallel gunzip {} ::: $output_dir/*.gz
+    cat $output_dir/* > $single_fasta
+    query=$single_fasta
+fi
+
+$SCRIPT_DIR/../../bin/fasta_split.py -l $batch_len -o $output_dir -c $qlen $query
 
 printf "Query file: %s\n" $query
 printf "Query length: %'d\n" `cat $qlen`
